@@ -46,6 +46,11 @@
 #	include <pwd.h>
 #endif
 
+using std::string;
+using std::vector;
+using std::ifstream;
+using std::ofstream;
+
 bool eqstr::operator()(const char *s1, const char *s2) const {
 	return strcmp(s1, s2) == 0;
 }
@@ -54,8 +59,6 @@ bool eqstr::operator()(const char *s1, const char *s2) const {
 	UCaseMap *ucm = NULL;
 #endif
 
-char buffer[20000];
-
 void sleep_ms(int milliseconds) {
 #ifdef _WIN32
 	Sleep(milliseconds);
@@ -139,19 +142,28 @@ string& remove_parenthesis(string &str) {
 
 string d2s(double value, int precision) {
 	// qgcvt(value, precision, buffer);
-	sprintf(buffer, "%.*G", precision, value);
+	char buffer[precision + 21];
+	snprintf(buffer, precision + 21, "%.*G", precision, value);
 	string stmp = buffer;
 	// gsub("e", "E", stmp);
 	return stmp;
 }
 
 string p2s(void *o) {
-	sprintf(buffer, "%p", o);
+	char buffer[21];
+	snprintf(buffer, 21, "%p", o);
 	string stmp = buffer;
 	return stmp;
 }
 string i2s(long int value) {
-	sprintf(buffer, "%li", value);
+	char buffer[21];
+	snprintf(buffer, 21, "%li", value);
+	string stmp = buffer;
+	return stmp;
+}
+string u2s(unsigned long int value) {
+	char buffer[21];
+	snprintf(buffer, 21, "%lu", value);
 	string stmp = buffer;
 	return stmp;
 }
@@ -220,12 +232,12 @@ size_t find_ending_bracket(const string &str, size_t start, int *missing) {
 char op2ch(MathOperation op) {
 	switch(op) {
 		case OPERATION_ADD: return PLUS_CH;
-		case OPERATION_SUBTRACT: return MINUS_CH;		
-		case OPERATION_MULTIPLY: return MULTIPLICATION_CH;		
-		case OPERATION_DIVIDE: return DIVISION_CH;		
-		case OPERATION_RAISE: return POWER_CH;		
+		case OPERATION_SUBTRACT: return MINUS_CH;
+		case OPERATION_MULTIPLY: return MULTIPLICATION_CH;
+		case OPERATION_DIVIDE: return DIVISION_CH;
+		case OPERATION_RAISE: return POWER_CH;
 		case OPERATION_EXP10: return EXP_CH;
-		default: return ' ';		
+		default: return ' ';
 	}
 }
 
@@ -419,7 +431,7 @@ void parse_qalculate_version(string qalculate_version, int *qalculate_version_nu
 }
 
 #ifdef _WIN32
-string utf8_encode(const wstring &wstr) {
+string utf8_encode(const std::wstring &wstr) {
 	if(wstr.empty()) return string();
 	int size_needed = WideCharToMultiByte(CP_UTF8, 0, &wstr[0], (int) wstr.size(), NULL, 0, NULL, NULL);
 	std::string strTo(size_needed, 0);
@@ -535,7 +547,7 @@ bool move_file(const char *from_file, const char *to_file) {
 
 	source.close();
 	dest.close();
-	
+
 	struct stat stats_from;
 	if(stat(from_file, &stats_from) == 0) {
 		struct utimbuf to_times;
@@ -543,9 +555,9 @@ bool move_file(const char *from_file, const char *to_file) {
 		to_times.modtime = stats_from.st_mtime;
 		utime(to_file, &to_times);
 	}
-	
+
 	remove(from_file);
-	
+
 	return true;
 #endif
 }
@@ -570,9 +582,12 @@ string getPackageDataDir() {
 }
 
 string getGlobalDefinitionsDir() {
-#ifndef WIN32
-	return string(PACKAGE_DATA_DIR) + "/qalculate";
+#ifdef COMPILED_DEFINITIONS
+	return "resource:///qalculate";
 #else
+#	ifndef WIN32
+	return string(PACKAGE_DATA_DIR) + "/qalculate";
+#	else
 	char exepath[MAX_PATH];
 	GetModuleFileName(NULL, exepath, MAX_PATH);
 	string datadir(exepath);
@@ -596,6 +611,7 @@ string getGlobalDefinitionsDir() {
 		return datadir + "\\data";
 	}
 	return datadir + "\\definitions";
+#	endif
 #endif
 }
 
@@ -694,7 +710,7 @@ char *locale_from_utf8(const char *str) {
 	char *dest, *buffer;
 	buffer = dest = (char*) malloc((outlength + 4) * sizeof(char));
 	if(!buffer) return NULL;
-	size_t err = iconv(conv, (char **) &str, &inlength, &buffer, &outlength);
+	size_t err = iconv(conv, (ICONV_CONST char **) &str, &inlength, &buffer, &outlength);
 	if(err != (size_t) -1) err = iconv(conv, NULL, &inlength, &buffer, &outlength);
 	iconv_close(conv);
 	memset(buffer, 0, 4);
@@ -709,7 +725,7 @@ char *locale_to_utf8(const char *str) {
 	char *dest, *buffer;
 	buffer = dest = (char*) malloc((outlength + 4) * sizeof(char));
 	if(!buffer) return NULL;
-	size_t err = iconv(conv, (char**) &str, &inlength, &buffer, &outlength);
+	size_t err = iconv(conv, (ICONV_CONST char**) &str, &inlength, &buffer, &outlength);
 	if(err != (size_t) -1) err = iconv(conv, NULL, &inlength, &buffer, &outlength);
 	iconv_close(conv);
 	memset(buffer, 0, 4 * sizeof(char));
@@ -726,6 +742,13 @@ char *utf8_strdown(const char *str, int l) {
 	if(!buffer) return NULL;
 	int32_t length = ucasemap_utf8ToLower(ucm, buffer, outlength, str, inlength, &err);
 	if(U_SUCCESS(err)) {
+		//basic accent removal
+		if(strcmp(buffer, "á") == 0 || strcmp(buffer, "à") == 0) {buffer[0] = 'a'; buffer[1] = '\0';}
+		else if(strcmp(buffer, "é") == 0 || strcmp(buffer, "è") == 0) {buffer[0] = 'e'; buffer[1] = '\0';}
+		else if(strcmp(buffer, "í") == 0 || strcmp(buffer, "ì") == 0) {buffer[0] = 'i'; buffer[1] = '\0';}
+		else if(strcmp(buffer, "ú") == 0 || strcmp(buffer, "ù") == 0) {buffer[0] = 'u'; buffer[1] = '\0';}
+		else if(strcmp(buffer, "ó") == 0 || strcmp(buffer, "ò") == 0 || strcmp(buffer, "õ") == 0) {buffer[0] = 'o'; buffer[1] = '\0';}
+		else if(strcmp(buffer, "ñ") == 0) {buffer[0] = 'n'; buffer[1] = '\0';}
 		return buffer;
 	} else if(err == U_BUFFER_OVERFLOW_ERROR) {
 		outlength = length + 4;
@@ -738,7 +761,6 @@ char *utf8_strdown(const char *str, int l) {
 			return buffer;
 		}
 	}
-	free(buffer);
 	return NULL;
 #else
 	return NULL;
@@ -775,7 +797,7 @@ int checkAvailableVersion(const char *version_id, const char *current_version, s
 #endif
 	res = curl_easy_perform(curl);
 	curl_easy_cleanup(curl);
-	curl_global_cleanup(); 
+	curl_global_cleanup();
 	if(res != CURLE_OK || sbuffer.empty()) {return -1;}
 	size_t i = sbuffer.find(version_id);
 	if(i == string::npos) return -1;
@@ -787,8 +809,8 @@ int checkAvailableVersion(const char *version_id, const char *current_version, s
 	if(s_version.empty()) return -1;
 	if(available_version) *available_version = s_version;
 	if(s_version != current_version) {
-		vector<int> version_parts_old, version_parts_new;
-		
+		std::vector<int> version_parts_old, version_parts_new;
+
 		string s_old_version = current_version;
 		while((i = s_old_version.find('.', 0)) != string::npos) {
 			version_parts_old.push_back(s2i(s_old_version.substr(0, i)));
@@ -800,7 +822,7 @@ int checkAvailableVersion(const char *version_id, const char *current_version, s
 			s_old_version = s_old_version.substr(i + 1);
 		}
 		version_parts_old.push_back(s2i(s_old_version));
-		
+
 		while((i = s_version.find('.', 0)) != string::npos) {
 			version_parts_new.push_back(s2i(s_version.substr(0, i)));
 			s_version = s_version.substr(i + 1);
@@ -811,14 +833,14 @@ int checkAvailableVersion(const char *version_id, const char *current_version, s
 			s_version = s_version.substr(i + 1);
 		}
 		version_parts_new.push_back(s2i(s_version));
-		
+
 		for(i = 0; i < version_parts_new.size(); i++) {
 			if(i == version_parts_old.size() || version_parts_new[i] > version_parts_old[i]) return true;
 			else if(version_parts_new[i] < version_parts_old[i]) return false;
 		}
-		
+
 	}
-	
+
 	return 0;
 #else
 	return -1;
